@@ -1,6 +1,6 @@
 // oscompatibility.h : LeechCore Windows/Linux compatibility layer.
 //
-// (c) Ulf Frisk, 2017-2022
+// (c) Ulf Frisk, 2017-2025
 // Author: Ulf Frisk, pcileech@frizk.net
 //
 #ifndef __OSCOMPATIBILITY_H__
@@ -15,18 +15,35 @@
 #include <setupapi.h>
 #include <conio.h>
 
-#define SOCK_NONBLOCK                       0
-
 #pragma warning( disable : 4477)
 
+#define SOCK_NONBLOCK                       0
 #define LC_LIBRARY_FILETYPE                 ".dll"
+#define libusb_device_handle                HANDLE
 #define LINUX_NO_OPTIMIZE
-VOID usleep(_In_ DWORD us);
+
+VOID BusySleep(_In_ DWORD us);
 
 #endif /* _WIN32 */
+
 #ifdef LINUX
+#define LC_LIBRARY_FILETYPE                 ".so"
 #include <libusb.h>
-#include <byteswap.h>
+// for some unexplainable reasons the gcc on -O2 will optimize out functionality
+// and destroy the proper workings on some functions due to an unexplainable
+// reason disable optimization on a function level resolves the issues ...
+#define LINUX_NO_OPTIMIZE __attribute__((optimize("O0")))
+#endif /* LINUX */
+
+#ifdef MACOS
+#define SOCK_NONBLOCK                       0
+#define LC_LIBRARY_FILETYPE                 ".dylib"
+#define libusb_device_handle                HANDLE
+#define LINUX_NO_OPTIMIZE
+#endif /* MACOS */
+
+#if defined(LINUX) || defined(MACOS)
+#define _FILE_OFFSET_BITS 64
 #include <ctype.h>
 #include <dirent.h>
 #include <dlfcn.h>
@@ -38,29 +55,31 @@ VOID usleep(_In_ DWORD us);
 #include <pthread.h>
 #include <time.h>
 #include <unistd.h>
-#include <sys/eventfd.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
-#define LC_LIBRARY_FILETYPE                 ".so"
-
 typedef void                                VOID, *PVOID;
 typedef void                                *HANDLE, **PHANDLE, *HMODULE, *FARPROC;
 typedef uint32_t                            BOOL, *PBOOL;
 typedef uint8_t                             BYTE, *PBYTE;
 typedef uint8_t                             UCHAR, *PUCHAR;
 typedef char                                CHAR, *PCHAR, *PSTR, *LPSTR;
+typedef const char                          *LPCSTR;
+typedef int32_t                             LONG;
 typedef uint16_t                            WORD, *PWORD, USHORT, *PUSHORT;
-typedef uint16_t                            WCHAR, *PWCHAR, *LPWSTR, *LPCWSTR;
-typedef uint32_t                            DWORD, *PDWORD, ULONG, *PULONG;
+typedef uint16_t                            WCHAR, *PWCHAR, *LPWSTR;
+typedef const uint16_t                      *LPCWSTR;
+typedef uint32_t                            UINT, DWORD, *PDWORD, ULONG, *PULONG;
 typedef long long unsigned int              QWORD, *PQWORD, ULONG64, *PULONG64;
 typedef uint64_t                            LARGE_INTEGER, *PLARGE_INTEGER, FILETIME;
 typedef size_t                              SIZE_T, *PSIZE_T;
 typedef void                                *OVERLAPPED, *LPOVERLAPPED;
 typedef struct tdEXCEPTION_RECORD32         { CHAR sz[80]; } EXCEPTION_RECORD32;
 typedef struct tdEXCEPTION_RECORD64         { CHAR sz[152]; } EXCEPTION_RECORD64;
+typedef int(*_CoreCrtNonSecureSearchSortCompareFunction)(void const *, void const *);
+#define __forceinline                       inline __attribute__((always_inline))
 #define TRUE                                1
 #define FALSE                               0
 #define MAX_PATH                            260
@@ -88,9 +107,15 @@ typedef struct tdEXCEPTION_RECORD64         { CHAR sz[152]; } EXCEPTION_RECORD64
 #define INVALID_SOCKET	                    -1
 #define SOCKET_ERROR	                    -1
 #define WSAEWOULDBLOCK                      10035L
-#define WAIT_OBJECT_0                       (0x00000000UL)
-#define INFINITE                            (0xFFFFFFFFUL)
 #define MAXIMUM_WAIT_OBJECTS                64
+#define WAIT_OBJECT_0                       (0x00000000UL)
+#define WAIT_FAILED                         (0xFFFFFFFFUL)
+#define WAIT_TIMEOUT                        (258L)
+#define INFINITE                            (0xFFFFFFFFUL)
+#define E_FAIL                              (0x80004005UL)
+#define error_status_t                      DWORD
+#define SecureZeroMemory(pb, cb)            (ZeroMemory(pb, cb))
+#define _itoa_s(n, s, l, b)                 (snprintf(s, l, "%d", n))
 
 #define _In_
 #define _In_z_
@@ -98,6 +123,7 @@ typedef struct tdEXCEPTION_RECORD64         { CHAR sz[152]; } EXCEPTION_RECORD64
 #define _Inout_
 #define _Inout_opt_
 #define _In_opt_
+#define _In_opt_z_
 #define _Out_opt_
 #define _Check_return_opt_
 #define _Frees_ptr_opt_
@@ -117,9 +143,15 @@ typedef struct tdEXCEPTION_RECORD64         { CHAR sz[152]; } EXCEPTION_RECORD64
 
 #define max(a, b)                           (((a) > (b)) ? (a) : (b))
 #define min(a, b)                           (((a) < (b)) ? (a) : (b))
-#define _byteswap_ushort(v)                 (bswap_16(v))
-#define _byteswap_ulong(v)                  (bswap_32(v))
-#define _byteswap_uint64(v)                 (bswap_64(v))
+#define _byteswap_ushort(v)                 (__builtin_bswap16(v))
+#define _byteswap_ulong(v)                  (__builtin_bswap32(v))
+#define _byteswap_uint64(v)                 (__builtin_bswap64(v))
+#ifndef _rotr
+#define _rotr(v,c)                          ((((DWORD)v) >> ((DWORD)c) | (DWORD)((DWORD)v) << (32 - (DWORD)c)))
+#endif /* _rotr */
+#define _rotr16(v,c)                        ((((WORD)v) >> ((WORD)c) | (WORD)((WORD)v) << (16 - (WORD)c)))
+#define _rotr64(v,c)                        ((((QWORD)v) >> ((QWORD)c) | (QWORD)((QWORD)v) << (64 - (QWORD)c)))
+#define _rotl64(v,c)                        ((QWORD)(((QWORD)v) << ((QWORD)c)) | (((QWORD)v) >> (64 - (QWORD)c)))
 #define _countof(_Array)                    (sizeof(_Array) / sizeof(_Array[0]))
 #define sprintf_s(s, maxcount, ...)         (snprintf(s, maxcount, __VA_ARGS__))
 #define strnlen_s(s, maxcount)              (strnlen(s, maxcount))
@@ -137,18 +169,19 @@ typedef struct tdEXCEPTION_RECORD64         { CHAR sz[152]; } EXCEPTION_RECORD64
 #define ExitThread(dwExitCode)              (pthread_exit(dwExitCode))
 #define ExitProcess(c)                      (exit(c ? EXIT_SUCCESS : EXIT_FAILURE))
 #define Sleep(dwMilliseconds)               (usleep(1000*dwMilliseconds))
-#define fopen_s(ppFile, szFile, szAttr)     ((*ppFile = fopen64(szFile, szAttr)) ? 0 : 1)
+#define fopen_s(ppFile, szFile, szAttr)     ((*ppFile = fopen(szFile, szAttr)) ? 0 : 1)
 #define GetModuleFileNameA(m, f, l)         (readlink("/proc/self/exe", f, l))
 #define ZeroMemory(pb, cb)                  (memset(pb, 0, cb))
 #define WinUsb_SetPipePolicy(h, p, t, cb, pb)   // TODO: implement this for better USB2 performance.
 #define WSAGetLastError()                   (WSAEWOULDBLOCK)    // TODO: remove this dummy when possible.
-#define _ftelli64(f)                        (ftello64(f))
-#define _fseeki64(f, o, w)                  (fseeko64(f, o, w))
+#define _ftelli64(f)                        (ftello(f))
+#define _fseeki64(f, o, w)                  (fseeko(f, o, w))
 #define _chsize_s(fd, cb)                   (ftruncate64(fd, cb))
 #define _fileno(f)                          (fileno(f))
-#define InterlockedAdd64(p, v)              (__sync_add_and_fetch(p, v))
-#define InterlockedIncrement64(p)           (__sync_add_and_fetch(p, 1))
+#define InterlockedAdd64(p, v)              (__sync_add_and_fetch_8(p, v))
+#define InterlockedIncrement64(p)           (__sync_add_and_fetch_8(p, 1))
 #define InterlockedIncrement(p)             (__sync_add_and_fetch_4(p, 1))
+#define InterlockedDecrement(p)             (__sync_sub_and_fetch_4(p, 1))
 #define GetCurrentProcess()					((HANDLE)-1)
 #define closesocket(s)                      close(s)
 
@@ -163,8 +196,11 @@ typedef struct tdCRITICAL_SECTION {
 VOID InitializeCriticalSection(LPCRITICAL_SECTION lpCriticalSection);
 VOID DeleteCriticalSection(LPCRITICAL_SECTION lpCriticalSection);
 VOID EnterCriticalSection(LPCRITICAL_SECTION lpCriticalSection);
+BOOL TryEnterCriticalSection(LPCRITICAL_SECTION lpCriticalSection);
 VOID LeaveCriticalSection(LPCRITICAL_SECTION lpCriticalSection);
 
+VOID BusySleep(_In_ DWORD us);
+
 typedef struct _SYSTEMTIME {
     WORD wYear;
     WORD wMonth;
@@ -224,11 +260,29 @@ HANDLE CreateEvent(_In_opt_ PVOID lpEventAttributes, _In_ BOOL bManualReset, _In
 DWORD WaitForMultipleObjects(_In_ DWORD nCount, HANDLE *lpHandles, _In_ BOOL bWaitAll, _In_ DWORD dwMilliseconds);
 DWORD WaitForSingleObject(_In_ HANDLE hHandle, _In_ DWORD dwMilliseconds);
 
-// for some unexplainable reasons the gcc on -O2 will optimize out functionality
-// and destroy the proper workings on some functions due to an unexplainable
-// reason disable optimization on a function level resolves the issues ...
-#define LINUX_NO_OPTIMIZE __attribute__((optimize("O0")))
-
+// SRWLOCK
+#ifdef LINUX
+typedef struct tdSRWLOCK {
+    uint32_t xchg;
+    int c;
+} SRWLOCK, *PSRWLOCK;
 #endif /* LINUX */
+#ifdef MACOS
+#include <dispatch/dispatch.h>
+typedef struct tdSRWLOCK {
+    union {
+        QWORD valid;
+        dispatch_semaphore_t sem;
+    };
+} SRWLOCK, *PSRWLOCK;
+#endif /* MACOS */
+VOID InitializeSRWLock(PSRWLOCK pSRWLock);
+VOID AcquireSRWLockExclusive(_Inout_ PSRWLOCK pSRWLock);
+VOID ReleaseSRWLockExclusive(_Inout_ PSRWLOCK pSRWLock);
+#define AcquireSRWLockShared    AcquireSRWLockExclusive
+#define ReleaseSRWLockShared    ReleaseSRWLockExclusive
+#define SRWLOCK_INIT            { 0 }
+
+#endif /* LINUX || MACOS */
 
 #endif /* __OSCOMPATIBILITY_H__ */
